@@ -33,6 +33,8 @@ TODO
 
 #include "nonposix.h"
 
+extern "C" const char *Version_string;
+
 static short int papersize = -1,  // papersize
 		 orientation = -1 , // orientation
 		 paperlength = 0, // Custom Paper size
@@ -151,7 +153,7 @@ void lbp_font::handle_unknown_font_command(const char *command,
 
 static void wp54charset()
 {
-  int i;
+  unsigned int i;
   
   lbpputs("\033[714;100;29;0;32;120.}");
   for (i = 0; i < sizeof(symset) ; i++) lbpputc(symset[i]);
@@ -561,7 +563,7 @@ static struct
 
 static int set_papersize(const char *papersize)
 {
-  int i;
+  unsigned int i;
 
   // First test for a standard (i.e. supported directly by the printer)
   // papersize
@@ -715,7 +717,6 @@ int main(int argc, char **argv)
 		  case 'l'  : 	orientation = 1;
 		  		break;
 		  case 'v'  :	{
-				extern const char *Version_string;
 				printf("GNU grolbp (groff) version %s\n",
 				       Version_string);
 				exit(0);
