@@ -106,11 +106,11 @@ struct node {
   virtual const char *type() = 0;
 };
 
-inline node::node() : next(0)
+inline node::node() : next(0), last(0)
 {
 }
 
-inline node::node(node *n) : next(n)
+inline node::node(node *n) : next(n), last(0)
 {
 }
 
@@ -181,9 +181,9 @@ public:
 };
 
 struct width_list {
-  width_list *next;
   hunits width;
   hunits sentence_width;
+  width_list *next;
   width_list(hunits, hunits);
   width_list(width_list *);
 };
