@@ -37,12 +37,13 @@ Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
 #include "nonposix.h"
 
+extern "C" const char *Version_string;
+
+#ifndef HAVE_MKSTEMP_PROTO
 extern "C" {
-  // Solaris 2.5.1 has these functions,
-  // but its stdlib.h fails to declare them.
-  char *mktemp(char *);
-  int mkstemp(char *);
+  extern int mkstemp(char *);
 }
+#endif
 
 #define DEFAULT_HASH_TABLE_SIZE 997
 #define TEMP_INDEX_TEMPLATE "indxbibXXXXXX"
@@ -180,7 +181,6 @@ int main(int argc, char **argv)
       break;
     case 'v':
       {
-	extern const char *Version_string;
 	printf("GNU indxbib (groff) version %s\n", Version_string);
 	exit(0);
 	break;
@@ -232,7 +232,8 @@ int main(int argc, char **argv)
   else
     name_max = dir_name_max(".");
   const char *filename = p ? p + 1 : basename;
-  if (name_max >= 0 && strlen(filename) + sizeof(INDEX_SUFFIX) - 1 > name_max)
+  if (name_max >= 0 &&
+      long(strlen(filename) + sizeof(INDEX_SUFFIX) - 1) > name_max)
     fatal("`%1.%2' is too long for a filename", filename, INDEX_SUFFIX);
   if (p) {
     p++;
@@ -502,7 +503,7 @@ static int do_file(const char *filename)
     // every CR character before a Newline.
     if (c == '\r') {
       int peek = getc(fp);
-      if (peek = '\n') {
+      if (peek == '\n') {
 	byte_count++;
 	c = peek;
       }
@@ -784,7 +785,7 @@ static void write_hash_table()
 
 static void fwrite_or_die(const void *ptr, int size, int nitems, FILE *fp)
 {
-  if (fwrite(ptr, size, nitems, fp) != nitems)
+  if (fwrite(ptr, size, nitems, fp) != (size_t)nitems)
     fatal("fwrite failed: %1", strerror(errno));
 }
 
